package parse_form;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_confvar parse_confform parse_servform);

#----------------------------------------------------------
# modul parse_form.pm
# part of webconf-package for eisfair
# $Id: parse_form.pm,v 1.2 2004/02/03 15:37:56 chriswue Exp $
# (c) 2002-2004 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl ':flock';
use formtools;
use check_form;

our $VERBOSE;

no strict "subs";
do "conf.pl";
use strict "subs";

my $config_path = "/etc/config.d";
my $form_path = "/var/install/form";

my $tag_group_on = '^[ \t]*<group ';
my $tag_group_off = '^[ \t]*<\/group>[ \t]*(#.*)?\s*$';

my $tag_option_on = '^[ \t]*<option ';
my $tag_option_off = '^[ \t]*<\/option>[ \t]*(#.*)?\s*$';

my $tag_form_on = '^[ \t]*<form ';
my $tag_form_off = '^[ \t]*<\/form>[ \t]*(#.*)?\s*$';

my $tag_subform_on = '^[ \t]*<subform ';
my $tag_subform_off = '^[ \t]*<\/subform>[ \t]*(#.*)?\s*$';

my $tag_empty = '^[ \t]*<empty\/>[ \t]*(#.*)?\s*$';
my $tag_comment_emptyline = '^[ \t]*#.*\s*$|^\s*$';

my $tag_text_on = '^[ \t]*<text>[ \t]*(#.*)?\s*$';
my $tag_text_off = '^[ \t]*<\/text>[ \t]*(#.*)?\s*$';

my @form;
my %var_hash;



# get_form description {{{
##############################################################################
# sub get_form($ff)
# - puts the content of $ff in an array, strips empty lines, lines which only
#   contains comments and \r\n and \n from the of each line
# IN:  $form_file - name of formfile relative to /var/install/form
# OUT: - modifies global array @form
##############################################################################
# }}}
# sub get_form($ff) {{{
sub get_form {
   my $form_file = shift;
   
   # read the formular
   open(FORM, "<$form_path/$form_file");

   while(<FORM>) {
      next if (/$tag_comment_emptyline/);
      s/\r?\n$//;
      push(@form, $_);
   }
   
   close(FORM);
}
# }}}



##############################################################################
#################### subroutines for config-file formular ####################
##############################################################################



# get_confvar description {{{
##########################################################
# sub get_confvar($cf)
# - reads the configuration variables from $cf
# IN:  $cf - name of configfile relative to /etc/config.d
# OUT: %hash (var-name => var-value)
##########################################################
# }}}
# sub get_confvar($cf) {{{
sub get_confvar {
   my $cf = shift;
   my ($name, $val);
   my %opt_hash;

   # read the content of the config-file
   open(CONF, "<$config_path/$cf");
   flock(CONF, LOCK_EX);

   # get configuration variables
   while(<CONF>) {
      next if (!/^[ \t]*([A-Z].*?)='([^']*)'[ \t]*(#.*)?\s*$/);
      $opt_hash{"$1"} = $2;
   }
   
   flock(CONF, LOCK_UN);
   close(CONF);

   return %opt_hash;
}
# }}}

# parse_confform description {{{
#####################################################################################
# sub parse_confform($cf, $ff)
# - creates the structure of the config-formular from $ff
# IN:  $cf - name of configfile
#      $ff - name of formfile
# OUT: $err  - $err = 0 if everything was ok, $err = 1 otherwise
#      $aref - reference on an array of complex hashes with the following structure:
#      @AoH = (
#               {
#                 name    => $group_name,
#                 options => [
#                              { name     => $optname,
#                                type     => $opttype,
#                                ro       => "yes|no",
#                                master   => $masteroption,
#                                match    => $regex,
#                                errormsg => $errormessage,
#                                val      => [ @slect_values ]
#                                file     => $configfile,
#                                ref      => $reference
#                              },
#                              ...
#                            ]
#               },
#               ...
#             )
#####################################################################################
# }}}
# sub parse_confform($cf, $ff) {{{
sub parse_confform {
   my ($cf, $ff) = @_;
   my @formular;
   my $err = 0;
   
   # get the form-file
   get_form($ff);
   %var_hash = get_confvar($cf);
  
   my ($line_count, $i, $line, $groupname) = ($#form, 0, "", "");

   while ($i <= $line_count) {
      $line = $form[$i++];      
      if    ($line =~ /$tag_empty/)    { push(@formular, {"name", "empty"}); }
      elsif ($line =~ /$tag_group_on/) { 
         $groupname = get_tag_opt($line, "name");
         my $aref = [];
         ($err, $i, $aref) = get_group($ff, $i, $aref);
         push(@formular, {"name", "$groupname", "options", [ @$aref ]});
      } else { $err = printErr($ff, $i, "'<group>' or '<empty/>'", "'$line'"); }
   }
   
   return ($err, \@formular);
}
# }}}

# get_group description {{{
##################################################################
# sub get_group($ff, $where, $aref)
# - creates an array of hashes containing the groups from @form
# IN:  $ff    - name of formfile
#      $where - actual position in @form
#      $aref  - a reference on an (empty) array
# OUT: $err   - $err = 0 if everything was ok, $err = 1 otherwise
#      $where - new position in @form
#      $aref  - reference on the array of hashes
##################################################################
# }}}
# sub get_group($ff, $where, $aref) {{{
sub get_group {
   my ($ff, $where, $aref) = @_;
   my ($eog, $err, $line, $msg, $fmsg) = (0, 0, "", "", "");

   while (!$eog) {
      $line = $form[$where++];
      if    ($line =~ /$tag_group_off/) { $eog = 1; }
      elsif ($line =~ /$tag_empty/)     { push (@$aref, {"name", "empty"}); }
      elsif ($line =~ /$tag_option_on/) {
         # get the option-tag variables
         my %tag_hash = get_tag_opt($line);

         ($err, $msg, $fmsg) = check_conf_option_tag(\%tag_hash, \%var_hash);
         $fmsg = "'$line'" if (!$fmsg);
         $err = printErr($ff, $where, $msg, "$fmsg") if ($err);
         
         my $opt_ref = {};
         
         # is option-tag closed?
         if (!$tag_hash{"end_of_tag"}) {
            # get the rest of the option-tags
            ($err, $where, $opt_ref) = get_confopt($ff, $where, \%tag_hash);

            ($err, $msg, $fmsg) = check_conf_option_hash($opt_ref);
            $fmsg = "'$line'" if (!$fmsg);
            $err = printErr($ff, $where, $msg, "$fmsg") if ($err);
         }
         else { $opt_ref = \%tag_hash; }
         # delete the eot-entry coming from get_tag_opt()
         delete($opt_ref->{"end_of_tag"});
         push(@$aref, { %$opt_ref });
      } else { $err = printErr($ff, $where, "a valid tag", "'$line'"); }
   }
    ### mark all masters
   my $k;
   my $l;
   my $prefix;
   my $optcnt = $#{ $aref };
   foreach $k (0 .. $optcnt) 
   {
      if ($aref->[$k]{"master"})
      {
         ($prefix) = split(/[^#]*$/, $aref->[$k]{"name"});
         foreach $l (0 .. $optcnt)
         {
            if ( $aref->[$k]{"master"} eq $aref->[$l]{"name"} )
            {
               $aref->[$l]{"ismaster"} = $prefix;
            }
         }
      }
   }

   
   return ($err, $where, $aref);
}
# }}}

# get_confopt description {{{
##################################################################
# sub get_confopt($ff, $where, $href)
# - creates a hash containing all tags for an option from @form
# IN:  $ff    - name of formfile
#      $where - actual position in @form
#      $href  - a reference on a hash
# OUT: $err   - $err = 0 if everything was ok, $err = 1 otherwise
#      $where - new position in @form
#      $href  - reference on the hash
##################################################################
# }}}
# sub get_confopt($ff, $where, $href) {{{
sub get_confopt {
   my ($ff, $where, $href) = @_;
   my ($eoo, $err, $line) = (0, 0, "");
   my @val;

   while (!$eoo) {
      $line = $form[$where++];
      if    ($line =~ /$tag_option_off/)                                     { $eoo = 1; }
      elsif ($line =~ /^[ \t]*<match>(.*?)<\/match>([ \t]*#.*?)?\s*$/)       { $href->{"match"} = "$1"; }
      elsif ($line =~ /^[ \t]*<errormsg>(.*?)<\/errormsg>([ \t]*#.*?)?\s*$/) { $href->{"errormsg"} = "$1"; }
      elsif ($line =~ /^[ \t]*<hint>(.*?)<\/hint>([ \t]*#.*?)?\s*$/)         { $href->{"hint"} = "$1"; }
      elsif ($line =~ /^[ \t]*<val>(.*?)<\/val>([ \t]*#.*?)?\s*$/)           { push(@val, $1); }
      elsif ($line =~ /^[ \t]*<file>(.*?)<\/file>([ \t]*#.*?)?\s*$/)         { $href->{"file"} = "$1"; }
      elsif ($line =~ /^[ \t]*<ref>(.*?)<\/ref>([ \t]*#.*?)?\s*$/)           { $href->{"ref"} = "$1"; }
      else { $err = printErr($ff, $where,  "'<match>', '<errormsg>', '<val>', '<file>' or '<ref>'", "'$line'"); }
   }

   # put val-tags into the hash if they exist
   if ($#val > 0 || $val[0]) { $href->{"val"} = [ @val ]; }
   return ($err, $where, $href);
}
# }}}



##############################################################################
###################### subroutines for service formular ######################
##############################################################################



# parse_servform description {{{
###############################################################################
# sub parse_servform($ff)
# - creates the structure of the service-formular from $ff
# IN:  $ff - name of formfile relative to /var/install/form
# OUT: $err  - $err = 0 if everything was ok, $err = 1 otherwise
#      $href - reference on a complex hash with with the following structure:
#      %HoA = (
#               formfile => $name_of_formfile,
#               name     => $formname,
#               prep     => $prepare_script,
#               help     => $helpfile,
#               subforms => [
#                             {
#                               name    => $subformname,
#                               button  => $button_name,
#                               script  => $apply_script,
#                               target  => $result_target,
#                               options => [
#                                            {
#                                              name         => $optname,
#                                              vname        => $varname,
#                                              hname        => $helptag_name,
#                                              type         => $opttype,
#                                              match        => $regex,
#                                              errormsg     => $errormessage,
#                                              size         => $optsize,
#                                              align        => $textalign,
#                                              source       => $sourcefile,
#                                              $sourcerange => $range
#                                              $item        => [ @item_list ],
#                                              $text        => $text
#                                            },
#                                            ...
#                                          ]
#                             },
#                             ...
#                           ]
#             )
###############################################################################
# }}}
# sub parse_servform($ff) {{{
sub parse_servform {
   my $ff = shift;
   my ($in_form, $eof, $err, $msg, $fmsg) = (0, 0, 0, "");
   my %form_hash;
   my @subforms;
 
   $form_hash{"formfile"} = $ff;
 
   # get the service-formfile
   get_form($ff);

   my ($line_count, $i, $line) = ($#form, 0, "");
  
   while($i <= $line_count && !$eof) {
      $line = $form[$i++];

      # got the <form ...>-tag?
      if (!$in_form) {
         # get the <form ...>-options (name, prep, help)
         if ($line =~ /$tag_form_on/) { 
            $in_form = 1;
            %form_hash = get_tag_opt($line);
            
            ($err, $msg, $fmsg) = check_serv_form_tag(\%form_hash);
            $fmsg = "'$line'" if (!$fmsg);
            $err = printErr($ff, $i, $msg, "$fmsg") if ($err);
            
            delete($form_hash{"end_of_tag"});
         } else { $err = printErr($ff, $i, "'<form>'", "'$line'"); }
      } else {
         if    ($line =~ /$tag_empty/)      { push(@subforms, {"name", "empty"}); }
         elsif ($line =~ /$tag_form_off/)   { $eof = 1; }
         elsif ($line =~ /$tag_subform_on/) {
            # get the <subform ...>-options (name, button, skript, target)
            my %sub_hash = get_tag_opt($line);

            ($err, $msg, $fmsg) = check_serv_subform_tag(\%sub_hash);
            $fmsg = "'$line'" if (!$fmsg);
            $err = printErr($ff, $i, $msg, "$fmsg") if ($err);
            
            my $aref = [];
            
            # get the options from subform
            ($err, $i, $aref) = get_subform($ff, $i, $aref);
            $sub_hash{"options"} = [ @$aref ];
            delete($sub_hash{"end_of_tag"});
            push(@subforms, { %sub_hash });
         } else { $err = printErr($ff, $i, "'</form>', '<subform>' or '<empty/>'", "'$line'"); }
      }
   }

   $form_hash{"subforms"} = [ @subforms ];
   return ($err, \%form_hash);
}
# }}}

# get_subform description {{{
##################################################################
# sub get_subform($ff, $where, $aref)
# - creates an array of hashes containing the subforms from $ff
# IN:  $ff    - name of formfile
#      $where - actual position in @form
#      $aref  - a reference on an array
# OUT: $err   - $err = 0 if everything was ok, $err = 1 otherwise
#      $where - new position in @form
#      $aref  - reference on the array
##################################################################
# }}}
# sub get_subform($ff, $where, $aref) {{{
sub get_subform {
   my ($ff, $where, $aref) = @_;
   my ($eosf, $err, $line, $msg, $fmsg) = (0, 0, "", "", "");

   while (!$eosf) {
      $line = $form[$where++];
      
      if    ($line =~ /$tag_empty/)       { push(@$aref, {"name", "empty"}); }
      elsif ($line =~ /$tag_subform_off/) { $eosf = 1; }
      elsif ($line =~ /$tag_option_on/)   {
         # get the <option ...>-options
         my %tag_hash = get_tag_opt($line);
         
         ($err, $msg, $fmsg) = check_serv_option_tag(\%tag_hash);
         $fmsg = "'$line'" if (!$fmsg);
         $err = printErr($ff, $where, $msg, "$fmsg") if ($err);
            
         my $opt_ref = {};
         
         # is <option ...>-tag closed?
         if (!$tag_hash{"end_of_tag"}) {
            # get the rest of the options-tags
            ($err, $where, $opt_ref) = get_servopt($ff, $where, \%tag_hash);
            
            ($err, $msg, $fmsg) = check_serv_option_hash($opt_ref);
            $fmsg = "'$line'" if (!$fmsg);
            $err = printErr($ff, $where, $msg, "$fmsg") if ($err);
         }
         else { $opt_ref = \%tag_hash; }
         delete($opt_ref->{"end_of_tag"});
         push(@$aref, { %$opt_ref });
      }
      else { $err = printErr($ff, $where, "'</subform>', '<option>' or '<empty/>'", "'$line'"); }
   }
   
   return ($err, $where, $aref);
}
# }}}

# get_servopt description {{{
##################################################################
# sub get_servopt($ff, $where, $href)
# - creates a hash containing all tags for an option from $ff
# IN:  $ff    - name of formfile
#      $where - actual position in @form
#      $href  - a reference on a hash
# OUT: $err   - $err = 0 is everything was ok, $err = 1 otherwise
#      $where - new position in @form
#      $href  - reference on the hash
##################################################################
# }}}
# sub get_servopt($ff, $where, $href) {{{
sub get_servopt {
   my ($ff, $where, $href) = @_;
   my ($eoo, $err, $text, $num, $line) = (0, 0, "", '[1-9]\d*', "");
   my @item;

   while (!$eoo) {
      $line = $form[$where++];

      if    ($line =~ /$tag_option_off/)                                                              { $eoo = 1; }
      elsif ($line =~ /^[ \t]*<match>(.*?)<\/match>[ \t]*(#.*)?\s*$/)                                 { $href->{"match"} = $1; }
      elsif ($line =~ /^[ \t]*<errormsg>(.*?)<\/errormsg>[ \t]*(#.*)?\s*$/)                           { $href->{"errormsg"} = $1; }
      elsif ($line =~ /^[ \t]*<hint>(.*?)<\/hint>[ \t]*(#.*)?\s*$/)                                   { $href->{"hint"} = $1; }
      elsif ($line =~ /^[ \t]*<size>(\d+|\d+x\d+|v|v\d+|h|h\d+)<\/size>[ \t]*(#.*)?\s*$/)             { $href->{"size"} = $1; }
      elsif ($line =~ /^[ \t]*<align>(?i)(left|right|center|block)<\/align>[ \t]*(#.*)?\s*$/)         { $href->{"align"} = $1; }
      elsif ($line =~ /^[ \t]*<source>(.*?)<\/source>[ \t]*(#.*)?\s*$/)                               { $href->{"source"} = $1; }
      elsif ($line =~ /^[ \t]*<sourcerange>(0|$num-x|$num-$num|$num)<\/sourcerange>[ \t]*(#.*)?\s*$/) { $href->{"sourcerange"} = $1; }
      elsif ($line =~ /^[ \t]*<item>(.*?)<\/item>[ \t]*(#.*)?\s*$/)                                   { push(@item, $1); }
      elsif ($line =~ /$tag_text_on/) {
         while (1) {
            $line = $form[$where++];
            last if ($line =~ /$tag_text_off/);
            $text .= "$line";
         }
      }
      else { $err = printErr($ff, $where,
                             "'</option>', '<match>', '<errormsg>', '<size>', '<align>', '<source>', '<sourcerange>', '<item>' or '<text>'",
                             "'$line'"); }
   }
   
   # if item-list exists, put it into the hash
   if ($#item > 0 || $item[0]) { $href->{"item"} = [ @item ]; }
   # if text exists, put it into the hash
   if ($text) { $href->{"text"} = $text; }
   
   return ($err, $where, $href);
}
# }}}

1;
