package renderer_html;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(create_renderer_html);


#-----------------------------------------------------------------------------
# modul renderer_html.pm
# part of the webconf-package for eisfair
# $Id: renderer_html.pm,v 1.4 2004/02/20 20:54:45 max Exp $
# (c) 2003-2004 by Maximilian Pasternak (maximilian.pasternak@bluescreen.de)
#-----------------------------------------------------------------------------

use strict;
use Fcntl ':flock';
use formtools;

our $VERBOSE;
our $CONF_LINE_LEN;
our $SERV_LINE_LEN;

no strict "subs";
do "conf.pl";
use strict "subs";


##############################################################################
# sub create_renderer_html()
# - prints html config-form
# IN:  $configfile - name of the config file
#      $classname  - name for the CSS-class
# OUT: renderer    - reference to hash with rendering functions
##############################################################################
sub create_renderer_html
{
   my $configfile = shift;
   my $matchpath  = "/var/config/tmp/.tmpformfile.$configfile";
   my $classname  = shift;

   ### build up hashtable for rendering configfiles
   my %renderer;

   ### open file
   open(MYFH, ">$matchpath") || die "Could not open '$matchpath', reason: $!\n";
   flock(MYFH, LOCK_EX);

   ### add the interface
   $renderer{"call"}          = html_call(\%renderer);
   $renderer{"print"}         = html_print(\%renderer);

   ### default will be used if no specific function is provided
   $renderer{"print_default"} = 
         html_print_line(\%renderer, $classname, $configfile);
   $renderer{"print_typeerror"} = 
         html_print_typeerror(\%renderer, $classname, $configfile);

   ### empty line
   $renderer{"print_empty"} = 
         html_print_empty(\%renderer, $classname, $configfile);

   ### header and footer
   $renderer{"print_header"} =
         html_print_header(\%renderer, $classname, $configfile);
   $renderer{"print_footer"} =
         html_print_footer(\%renderer, $classname, $configfile);
   $renderer{"is_group"} = "false";

   ### groups
   $renderer{"begin_group"} =
         html_begin_group(\%renderer, $classname, $configfile);
   $renderer{"end_group"} =
         html_end_group(\%renderer, $classname, $configfile);

   ### lists
   $renderer{"begin_list"} =
         html_print_empty(\%renderer, $classname, $configfile);
   $renderer{"next_list"} =
         html_print_empty(\%renderer, $classname, $configfile);
#   $renderer{"end_list"} =
#         html_print_empty(\%renderer, $classname, $configfile);

   ### config items
   $renderer{"print_line"} =
         html_print_line(\%renderer, $classname, $configfile);
   $renderer{"print_yesno"} =
         html_print_yesno(\%renderer, $classname, $configfile);
   $renderer{"print_select"} =
         html_print_select(\%renderer, $classname, $configfile);
   $renderer{"print_ref"} =
         html_print_ref(\%renderer, $classname, $configfile);

   ### close renderer
   $renderer{"close"} = html_close(\%renderer);

   ### read helpfile
   if (-f "/var/install/help/$configfile")
   {
      my $helptopic = "";
      my $helptext  = "";
      my %helphash;
      $renderer{"help"}       = \%helphash;

      my $Tag_help_on = "^[ \t]*<help name=\"(.*?)\">";
      my $Tag_help_off = '^[ \t]*<\/help>[ \t]*(#.*?)?\s*$';

      open(HELP, "</var/install/help/$configfile");
      LINE: while(<HELP>)
      {
         if (/$Tag_help_on/)
         {
            $helptopic = "$1";
            next LINE;
         }

         if ($helptopic)
         {
            if (/$Tag_help_off/)
            {
               $helphash{$helptopic} = $helptext;
               $helptopic = "";
               $helptext  = "";
               next LINE;
            }

            $helptext .= $_;
         }
      }
      close(HELP);
   }

   ### return renderer
   return \%renderer
}


##############################################################################
# sub renderer.call()
# - interface for calling the renderer
# IN:  $renderer  - this-pointer
#      $function  - function to be called
#      $param     - parameter for the function
# OUT: -
##############################################################################

sub html_call
{
my $renderer = shift;
return sub
{
   ### interface for this renderer
   my $function = shift || "";
   my $param    = shift || "";

   $renderer->{$function}->($param) if ($renderer->{$function});
}
}


##############################################################################
# sub renderer.print()
# - interface for printing elements
# IN:  $renderer  - this-pointer
#      $type      - type to render
#      $name      - name of the element
#      $value     - value of the element
#      $param     - hash with parameters
# OUT: -
##############################################################################

sub html_print
{
my $renderer = shift;
return sub
{
   ### interface for this renderer
   my $type     = shift || "unknown";
   my $name     = shift || "unknown";
   my $value    = shift;
   my $param    = shift;

   my $function = $renderer->{"print_$type"};

   if (!$function && $VERBOSE eq "yes")
   {
      $name      = "&lt$type&gt $name = '$value'";
      $function  = $renderer->{"print_typeerror"};
   }
   if (!$function)
   {
      $function  = $renderer->{"print_default"};
   }

   $function->($name, $value, $param)           if ($function);
}
}


##############################################################################
#
# implementation of the renderer
#
#############################################################################

sub html_close
{
my $renderer  = shift;
return sub
{
   flock(MYFH, LOCK_UN);
   close(MYFH);
}
}


sub html_print_header
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift;
return sub
{
   # print the head of the config
   print_head("config-file formular", 
              "config for $service", 
              "class=\"$classname\"", 1);
   add_history("$service configuration#make_forms.pl?$service");
   print_history;
   print "<hr class=\"$classname\">\n";
   print "<form class=\"$classname\" ";
   print "action=\"choose_action.pl?$service+.tmpformfile.$service\" ";
   print "method=\"post\" target=\"Result\">\n";
}
}


sub html_print_footer
{
my $renderer  = shift;
my $classname = shift;
return sub
{
   my $timestamp = localtime(time);

   print "<br><input class=\"button-$classname\" ";
   print "type=\"submit\" value=\" update \">\n";
   print "</form>\n";
   print "<hr class=\"$classname\">\n";
   print_history;
   print "<hr class=\"$classname\">\n";
   print "<span class=\"$classname\">created by renderer_html.pm on $timestamp.</span>\n" if ($VERBOSE eq "yes");
   print "</body></html>\n";
#   print "<br>return to <a class=\"$classname\" ";
#   print "href=\"choose_design.pl?adm\" target=\"_parent\">Servicemenu</a>\n";
#   print "<br>return to <a class=\"$classname\" ";
#   print "href=\"../index.html\" target=\"_parent\">Mainmenu</a>\n";

#   print "<br><hr class=\"$classname\">\n";
#   print "<span class=\"$classname\">created by renderer_html.pm on $timestamp.</span>\n" if ($VERBOSE eq "yes");
#   print "</body></html>\n";
}
}

sub html_begin_group
{
my $renderer  = shift;
my $classname = shift;
return sub
{
   my $name  = shift;

   print "<fieldset class=\"$classname\">";
   print "<legend class=\"$classname\">$name</legend>\n"     if ($name);
   print "<BR><table class=\"$classname\">\n";

   $renderer->{"is_group"} = "true";
}
}


sub html_end_group
{
my $renderer  = shift;
my $classname = shift;
return sub
{
   my $name  = shift;

   print "</table>";
   print "</fieldset>";
   print "\n";

   $renderer->{"is_group"} = "false";
}
}


sub html_print_empty
{
my $renderer  = shift;
return sub
{
   if ($renderer->{"is_group"} =~ /^true$/)
   { 
      print "<tr><td><br></td></tr>\n";
   }
   else
   {
      print "<p></p>\n";
   }
}
}


sub html_print_typeerror
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift || "";
return sub
{
   my $name   = shift;

   print "<tr class=\"$classname\">\n";
   print "<td class=\"optname-$classname\"><b>";
   print "ERROR: invalid option:</b></td>\n";
   print "<td class=\"opt-$classname\">$name</td>\n";
   print "</tr>\n";
}
}


sub html_print_line
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift || "";
return sub
{
   my ($name, $value, $hstruct) = @_;
   my ($oname, $hint, $rostr, $length, $match, $err) = "";

   $oname  = $hstruct->{"name"} || "" if ($hstruct);
   $hint   = $hstruct->{"hint"} || "" if ($hstruct);
   $hint   = $renderer->{"help"}->{$oname} || "" if ($hstruct && !$hint);

   $rostr  = $hstruct->{"ro"}    || "" if ($hstruct);
   $rostr = "readonly"    if $rostr eq "yes";
   $length = ($rostr eq "readonly") ? length($value) : $CONF_LINE_LEN;

   $match  = $hstruct->{"match"} || "" if ($hstruct);
   $err    = $hstruct->{"errormsg"} || "Value '\%v' doesn't match '$match'" if ($hstruct);
			

   print "<tr class=\"$classname\">\n";
   print "<td class=\"optname-$classname\">$name = </td>\n";
   print "<td class=\"opt-$classname\">";
   print "<input class=\"text-$classname\" name=\"$name\" ";
   print "type=\"text\" size=\"$length\" ";
   print "title=\"$hint\" "     if ($hint);
   print "maxlength=\"255\" value=\"$value\" $rostr>";

   ### helptopic found
   if ($renderer->{"help"}->{$oname})
   {
      my $o = $oname;
      $o =~ s/#/%23/g;
      print " &nbsp;&nbsp;<a class=\"$classname\" ";
      print "href=\"show_help.pl?$service+$o\" target=\"Result\">Help</a>";
   }
   print "</td></tr>\n";

   print MYFH "$name=$match\n";
   print MYFH "$name-err=$err\n";
}
}


sub html_print_yesno
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift || "";
return sub
{
   my $name   = shift;
   my $value  = shift;

   my $oname  = @_[0]->{"name"} || "";
   my $length = length($value);

   my $hint   = @_[0]->{"hint"} || "";
   $hint = $renderer->{"help"}->{$oname} || "" unless ($hint);

   my $ro     = @_[0]->{"ro"}   || "";

   print "<tr class=\"$classname\">\n";
   print "<td class=\"optname-$classname\">$name = </td>\n";
   print "<td class=\"opt-$classname\">";
   if ($ro eq "yes")
   {
      print "<input class=\"text-$classname\" name=\"$name\" ";
      print "type=\"text\" size=\"$length\" ";
      print "title=\"$hint\" "     if ($hint);
      print "maxlength=\"$length\" value=\"$value\" readonly>";
   } else {
      print "<select class=\"$classname\" name=\"$name\" size=\"1\"";
      print " title=\"$hint\" "     if ($hint);
      print ">\n";
      my $sel_y = $value =~ /yes/i ? "selected" : "";
      my $sel_n = $sel_y ? "" : "selected";
      print "<option class=\"$classname\" $sel_y>yes</option>\n";
      print "<option class=\"$classname\" $sel_n>no</option>\n";
      print "</select>";
   }

   ### helptopic found
   if ($renderer->{"help"}->{$oname})
   {
      my $o = $oname;
      $o =~ s/#/%23/g;
      print " &nbsp;&nbsp;<a class=\"$classname\" ";
      print "href=\"show_help.pl?$service+$o\" target=\"Result\">Help</a>";
   }
   print "</td></tr>\n";
}
}

sub html_print_select
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift || "";
return sub
{
   my $name   = shift;
   my $value  = shift;
   my $valref = @_[0]->{"val"}  || "";

   my $oname  = @_[0]->{"name"} || "";

   my $hint   = @_[0]->{"hint"} || "";
   $hint = $renderer->{"help"}->{$oname} || "" unless ($hint);

   print "<tr class=\"$classname\">\n";
   print "<td class=\"optname-$classname\">$name = </td>\n";
   print "<td class=\"opt-$classname\">";

   print "<select class=\"$classname\" name=\"$name\" size=\"1\"";
   print " title=\"$hint\" "     if ($hint);
   print ">\n";

   foreach (@$valref)
   {
      print "<option class=\"$classname\"";
      print " selected"                       if (/$value/);
      print ">$_</option>\n";
   }
   print "</select>\n";

   ### helptopic found
   if ($renderer->{"help"}->{$oname})
   {
      my $o = $oname;
      $o =~ s/#/%23/g;
      print " &nbsp;&nbsp;<a class=\"$classname\" ";
      print "href=\"show_help.pl?$service+$o\" target=\"Result\">Help</a>";
   }
   print "</td></tr>\n";
}
}

sub html_print_ref
{
my $renderer  = shift;
my $classname = shift;
my $service   = shift || "";
return sub
{
   my $name   = shift;
   my $value  = shift;
   my @valref;

   my $oname  = @_[0]->{"name"} || "";
   my $length = length($value) + 1;

   my $hint   = @_[0]->{"hint"} || "";
   $hint = $renderer->{"help"}->{$oname} || "" unless ($hint);

   my $ro     = @_[0]->{"ro"}   || 0;

   my $match  = @_[0]->{"match"} || "";
   my $err    = @_[0]->{"errormsg"} || "Value '\%v' doesn't match '$match'";

   my $sname = "/etc/config.d/".@_[0]->{"file"} || "";
   $sname = ""          if (!-f $sname);
   my $m     = @_[0]->{"ref"}                   || "";
   if ($sname && $m)
   {
      my $count = 0;
      open(SOURCE, "<$sname");
      while (<SOURCE>) { $count = $1 if (/^$m='([^']+)'/); }
      close(SOURCE);

      for (1 .. $count) { push(@valref, $_); }
   }
   else
   {
      $value = "No reference specified!"      if (!$m);
      $value = "Reference is zero!"           if ($m == 0);
      $value = "No configfile specified!"     if (!$sname);
      $value = "No configfile and no reference specified!" if (!$sname && !$m);
      $ro    = "yes";
      $match = '#dec_0_s#';
   }

   print "<tr class=\"$classname\">\n";
   print "<td class=\"optname-$classname\">$name = </td>\n";
   print "<td class=\"opt-$classname\">";

   if ($ro eq "yes")
   {
      $length = length($value);
      print "<input class=\"text-$classname\" name=\"$name\" ";
      print "title=\"$hint\" "     if ($hint);
      print "type=\"text\" size=\"$length\"";
      print "maxlength=\"$length\" value=\"$value\" readonly>";
   }
   else
   {
      print "<select class=\"$classname\" name=\"$name\" size=\"1\"";
      print " title=\"$hint\" "     if ($hint);
      print " multiple";
      print ">\n";

      foreach (@valref)
      {
         print "<option class=\"$classname\"";
         print " selected"                       if ( $value =~ /$_/ );
         print ">$_</option>\n";
      }
      print "</select>\n";
   }

   ### helptopic found
   if ($renderer->{"help"}->{$oname})
   {
      my $o = $oname;
      $o =~ s/#/%23/g;
      print " &nbsp;&nbsp;<a class=\"$classname\" ";
      print "href=\"show_help.pl?$service+$o\" target=\"Result\">Help</a>";
   }
   print "</td></tr>\n";

   if ($match)
   {
      print MYFH "$name=$match\n";
      print MYFH "$name-err=\n";
   }
}
}

1;

